INCIDENTS_TYPES = [
    'Malware', 'Phishing', 'Access Violation', 'Lateral Movement', 'Port Scan', 'Sql Injection', 'Brute Force',
    'Control Avoidance', 'Rogue Device', 'Denial Of Service', 'Account Compromised'
]
SEVERITIES = ['Low', 'Medium', 'High', 'Critical']
